import json
import time
from urllib.parse import urlencode
import requests
import schemathesis
from schemathesis.core.transport import Response as SchemathesisResponse
from rest_framework.response import Response as DRFResponse
from django.core.handlers.wsgi import WSGIRequest

from schemathesis.transport.wsgi import (
    WSGI_TRANSPORT,
    WSGITransport,
    REQUESTS_TRANSPORT,
)


class Transport(WSGITransport):
    def __init__(self):
        super().__init__()
        self._copy_serializers_from(WSGI_TRANSPORT)

    @staticmethod
    def case_as_request(case):
        if isinstance(case, schemathesis.Case):
            r_dict = REQUESTS_TRANSPORT.serialize_case(
                case,
                base_url=case.operation.base_url,
            )
            return requests.Request(**r_dict).prepare()
        if isinstance(case, WSGIRequest):
            return requests.Request(
                method=case.method,
                url=case.build_absolute_uri(),
                headers=case.headers,
                data=case.POST,
                files=case.FILES,
                cookies=case.COOKIES,
                params=case.GET,
            ).prepare()

        return case

    def send(self, case: schemathesis.Case, *args, **kwargs):
        t = time.time()
        case.headers.pop("Authorization", "")
        serialized_request = WSGI_TRANSPORT.serialize_case(case)
        serialized_request.update(
            QUERY_STRING=urlencode(serialized_request["query_string"]),
        )
        if json_data := serialized_request.pop("json", None):
            serialized_request.update(data=json.dumps(json_data))
        import django.test

        client = django.test.Client()
        response: DRFResponse = client.generic(**serialized_request)
        elapsed = time.time() - t
        return self.get_st_response(response, case, elapsed)

    @classmethod
    def get_st_response(self, response: DRFResponse, case=None, elapsed=1):
        if not case:
            case = response.wsgi_request
        return SchemathesisResponse(
            response.status_code,
            headers={k: [v] for k, v in response.headers.items()},
            content=response.content,
            request=self.case_as_request(case),
            elapsed=elapsed,
            verify=True,
        )



import json
import logging
import os
import time
from collections import namedtuple
from urllib.parse import urlencode
from .h4f import FatalError, PostDict, fetch_page_with_retries
from history4feed.app.settings import history4feed_server_settings as settings
import requests
from datetime import UTC, datetime as dt
import time
import requests
from datetime import datetime as dt, date, timedelta
from dateparser import parse as parse_date
DEFAULT_USER_AGENT = "curl"

class SearchIndexError(FatalError):
    pass

def fetch_posts_links_with_serper(site, from_time: dt, to_time: dt = None, delta_days=100) -> dict[str, PostDict]:
    s = requests.Session()
    s.headers.update({
        'X-API-KEY':  os.getenv("SERPER_API_KEY"),
        'Content-Type': 'application/json'
    })

    params = dict(num=100, page=1)
    entries: dict[str, PostDict] = {}
    to_time = to_time or dt.now(UTC)
    if not to_time.tzinfo:
        to_time = to_time.replace(tzinfo=UTC)

    frame_start = from_time - timedelta(days=1)
    credits_used = 0

    while frame_start < to_time:
        frame_end = frame_start + timedelta(days=delta_days)
        params.update(q=f"site:{site} after:{frame_start.date().isoformat()} before:{frame_end.date().isoformat()}", page=1)
        while True:
            resp = s.get("https://google.serper.dev/search", params=params)
            if not resp.ok:
                raise SearchIndexError(f"Serper Request GOT {resp.status_code}: {resp.text}")
            data = resp.json()
            credits_used += data['credits']
            for d in data['organic']:
                date = d.get('date')
                if date:
                    date = parse_date(date)
                else:
                    date = min(frame_end, to_time)
                post = PostDict(link=d['link'], title=d['title'], pubdate=date, categories=[])
                entries[post.link] = post
            params['page'] += 1
            if len(data['organic']) < params['num']:
                break
        frame_start = frame_end - timedelta(days=1)
    logging.info(f"got {len(entries)} posts between {from_time} and {to_time}, used {credits_used} credits")
    return entries




# Generated by Django 5.1.6 on 2025-05-02 14:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('history4feed', '0004_alter_fulltextjob_status_alter_job_state'),
    ]

    operations = [
        migrations.AddField(
            model_name='feed',
            name='datetime_modified',
            field=models.DateTimeField(default=None, help_text='date feed entry was edited in the database', null=True),
        ),
    ]



"""
URL configuration for history4feed project.

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/5.0/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.contrib import admin
from django.urls import include, path
from .app import views
from rest_framework import routers
from drf_spectacular.views import SpectacularAPIView, SpectacularRedocView, SpectacularSwaggerView


from django.http import JsonResponse
def handler404(*args, **kwargs):
    return JsonResponse(dict(code=404, message='non-existent page'), status=404)

def handler500(*args, **kwargs):
    return JsonResponse(dict(code=500, message='internal server error'), status=500)


API_VERSION = "v1"

router = routers.SimpleRouter(use_regex_path=False)
router.register("feeds", views.FeedView, "feed-view")
router.register("feeds/<uuid:feed_id>/posts", views.FeedPostView, "feed-post-view")
router.register("feeds", views.RSSView, "feed-rss-view")
router.register("posts", views.PostOnlyView, "post-view")
router.register("jobs", views.JobView, "job-view")


urlpatterns = [
    path(f'api/{API_VERSION}/', include(router.urls)),
    path('admin/', admin.site.urls),

    # YOUR PATTERNS
    path('api/schema/', views.SchemaViewCached.as_view(), name='schema'),
    # Optional UI:
    path('api/schema/swagger-ui/', SpectacularSwaggerView.as_view(url_name='schema'), name='swagger-ui'),
]
